@@ -5,7 +5,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2017	PgPool Global Development Group
+ * Copyright (c) 2003-2023	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -22,8 +22,7 @@
  */
 #include "pool.h"
 
-int debug = 0;	/* non 0 if debug option is given (-d). pcp only */
-pid_t mypid;	/* pgpool parent process id */
-bool run_as_pcp_child;
+pid_t		mypid;				/* pgpool parent process id */
 ProcessType processType;
 ProcessState processState;
+bool		reset_query_error;	/* true if error returned from backend while processing reset queries */
